% This file uses mat files to produce the data for figures 1-3
% Ludwig Straub and Ivan Werning, 2019
%
% This file requires make_mat_files to have been run and csvwrite_with_headers.m
%
% For each figure number 'figno', this file produces data stored in a csv,
% as well as four figures:
% figure(11): paths in (k,C) state space
% figure(12): time paths of k
% figure(13): time paths of C
% figure(14): time paths of taxes
%
% In case of figno == 2, the data is shown in the Matlab Command window,
% and can then be pasted into the tikz file directly.

clear all
close all

% Select which figure to produce data for
figno = 3;

% legend identifier
legend_id = 'sigma = ';

if figno == 1
    param_all = [0.75,0.9,0.95,0.99,1.025,1.05,1.1,1.25];
elseif figno == 2
    param_all = [0.5 0.525 0.55 0.575 0.6 0.625 0.65 0.675 0.7 0.725 0.75 0.775 0.8...
        0.81 0.82 0.825 0.83 0.84 0.85 0.86 0.87 0.875 0.88 0.89 ...
        0.9 0.905 0.91 0.915 0.92 0.925 0.93 0.935 0.94 0.945 0.95 0.955 0.96 0.961 0.962 0.963 0.964 0.965 0.967 0.968 0.969 0.97];
elseif figno == 3
    param_all = [-0.8:0.2:0.8];
    legend_id = 'redist = ';
end


%%% Use smaller grid?
small_grid = 0;


% On which grids was the data generated?
ngridk  = 512;
ngridC  = 300;
if small_grid==1, ngridC = 100; ngridk = 128; end 

% Set time in the plots
TT = 300;
if figno == 2, TT      = 2000; end  % need more time for the speed plot

KTIME   = zeros(TT,length(param_all));
CTIME   = KTIME;
TAXES   = KTIME;

for i_p = 1:length(param_all)
    
    % read out parameters
    if figno == 3 % in this case, redist varies within for loop
        sigma = 1.25;
        redist = param_all(i_p);
    else
        sigma = param_all(i_p);
        redist = 1;
    end
    iter_self_g = 10;
    sigma_w = sigma;
    
    % set filepath
    filepath = ['fig' num2str(figno) '/'];
    
    if figno == 1
        khigh = 3;
        if sigma < 1
            klow = 0.84352;
        elseif sigma > 1
            klow = 0.012581;
        end
        g = 0.26718;
        
        filename = num2str([filepath,'judd_','sigma_',num2str(sigma),'redist',num2str(redist),...
            'sigma_w_',num2str(sigma_w),'_grid',num2str(ngridk),'x',num2str(ngridC),...
            'range_',num2str(klow),'-',num2str(khigh),...
            'g_', num2str(g),...
            'iter_sg_',num2str(iter_self_g),...
            '.mat']);

    elseif figno == 2
        klow = 1.3129;
        khigh = 3;
        g = 0.26718;
        
        redist = 1;
        ngridC = 200;
        ngridk = 256;
        if small_grid==1, ngridC = 100; ngridk = 128; end 

        filename = num2str([filepath,'judd_','sigma_',num2str(sigma),'redist',num2str(redist),...
            'sigma_w_',num2str(sigma_w),'_grid',num2str(ngridk),'x',num2str(ngridC),...
            'range_',num2str(klow),'-',num2str(khigh),...
            'g_', num2str(g),...
            'iter_sg_',num2str(iter_self_g),...
            '.mat']);
        
    elseif figno == 3
        klow = 0.012581;
        khigh = 3;
        if redist < 0
            khigh = 10;
        end
        g = 0.26718;
        
        filename = num2str([filepath,'judd_','sigma_',num2str(sigma),'redist',num2str(redist),...
            'sigma_w_',num2str(sigma_w),'_grid',num2str(ngridk),'x',num2str(ngridC),...
            'range_',num2str(klow),'-',num2str(khigh),...
            'g_', num2str(g),...
            'iter_sg_',num2str(iter_self_g),...
            '.mat']);
        
    end
    
    load(filename,'-regexp','^(?!figno)\w')
    
    %% path basics
    % use this if an external file has been loaded
    %sigma_i = 1;
    %SIGMA_all = [sigma];
    
    T = TT;
    time = [1:T];
    ktime   = zeros(T,1);
    Ctime   = ktime;
    psitime = ktime;
    taxes   = zeros(T-1,1);

    % functions
    psi_inv  = @(k,psi) interp1(kk,CC(1,:),k)  +  ( interp1(kk,CC(end,:),k)-interp1(kk,CC(1,:),k) ) * psi;
    Rbar_fun = @(C, C_) 1/beta* (C_/C).^(-sigma);
    tax_fun  = @(C, C_, kprime)  1 - Rbar_fun(C, C_)./ (alpha * kprime.^(alpha-1) + (1-delta));
    k0       = kstar1;
    
    % initial values
    ktime(1,1)   = k0;
    psitime(1,1) = fminbnd(@(psiC) -interp2(kk,psi_C, V, ktime(1,1), psiC),0,1);
    Ctime(1,1)   = psi_inv(ktime(1,1),psitime(1,1));
    taxes(1,1)   = 0;
    %Ctime(1,1)   = 1.002*ktime(1,1)*(1-beta)/beta;
    %psitime(1,1) = (Ctime(1,1)-interp1(kk,CC(1,:),ktime(1,1)))/(interp1(kk,CC(end,:),ktime(1,1))-interp1(kk,CC(1,:),ktime(1,1)));

    for t=1:T-1
        psitime(t+1) = interp2(kk,psi_C, policy_psi, ktime(t),psitime(t));
        ktime(t+1)   = interp2(kk,psi_C, policy_k, ktime(t),psitime(t));
        Ctime(t+1)   = psi_inv(ktime(t+1),psitime(t+1));
        taxes(t)     = tax_fun(Ctime(t+1), Ctime(t), ktime(t+1));
    end
    
    % write the paths into the matrices
    KTIME(:,i_p) = ktime;
    CTIME(:,i_p) = Ctime;
    TAXES(:,i_p) = [taxes; taxes(end)];
    
    % do the plotting
    if figno == 1 || figno == 3
        figure(11); hold on
        stateplot(i_p) = plot(ktime, Ctime);

        figure(12); hold on
        kplot(i_p) = plot(time, ktime);

        figure(13); hold on
        Cplot(i_p) = plot(time, Ctime);

        figure(14); hold on
        % plot every mth data point
        m = 4;
        taxplot(i_p) = plot(time(1:m:end-1), taxes(1:m:end));
    end
    
end


%% design plots
if figno == 1 || figno == 3

    Nsig = length(param_all);
    colors = hot(2*length(param_all));

    Cstar1 = (1-beta)/beta * kstar1; % compute Judd steady state C consumption

    % Set path where figures are stored
    printpath = [filepath 'plots/'];

    % Write csv
    if 1==1
        csvpath   = [filepath 'csv/'];
        csvname   = 'data';
        
        % construct header
        theader   = 't  ';
        kheader   = 'k1 ';
        Cheader   = 'C1 ';
        taxheader = 't1 ';
        for i_p = 2:length(param_all)
            if i_p <= 9
                sigstring = [num2str(i_p) ' '];
            else
                sigstring = num2str(i_p);
            end
            kheader     = [kheader; 'k' sigstring];
            Cheader     = [Cheader; 'C' sigstring];
            taxheader   = [taxheader; 't' sigstring];
        end
        Header = cellstr([theader; kheader; Cheader; taxheader])';
        
        % write csv file
        csvwrite_with_headers([csvpath csvname '.csv'], [time' KTIME CTIME TAXES],Header);
    end

    filetype = 'png'; % alternative: eps
    printtype = '-dpng'; % alternative: d-epsc to print eps in color
    resolution = '-r900'; % set resolution

    line_width = 1;

    % consider one after each other
    % --- state space plot ---
    figure(11)
    hold on
    ttext = title('paths in (k,C) state space');
    xtext = xlabel('k');
    ytext = ylabel('C');
    grid on

    legendcell = {};
    for i = 1:Nsig
        set(stateplot(i), ...
            'Color', colors(i,:), ...
            'LineWidth', line_width, ...
            'LineStyle', '-');
        legendcell{i} = strcat(legend_id,num2str(param_all(i)));
    end
    legnd = legend(legendcell);
    set(xtext, ...
        'FontName', 'Palatino', ...
        'FontSize', 12 );
    set(ytext, ...
        'FontName', 'Palatino', ...
        'FontSize', 12 );
    set(ttext, ...
        'FontName', 'Palatino', ...
        'FontSize', 16, ...
        'FontWeight', 'bold');
    set(legnd, ...
        'FontName', 'Palatino', ...
        'FontSize', 12,...
        'Location', 'NorthWest',...
        'Box','off');
    filename = 'statespace';
    print(printtype, strcat(printpath, filename, '.', filetype),resolution);

    % --- k plot ---
    figure(12)
    hold on
    ttext = title('paths of k over time');
    xtext = xlabel('time');
    ytext = ylabel('k');
    line = plot(time, kstar1 + 0*time,'k--'); % steady state line
    grid on

    for i = 1:Nsig
        set(kplot(i), ...
            'Color', colors(i,:), ...
            'LineWidth', line_width, ...
            'LineStyle', '-');
    end
    legnd = legend(legendcell);
    set(xtext, ...
        'FontName', 'Palatino', ...
        'FontSize', 12 );
    set(ytext, ...
        'FontName', 'Palatino', ...
        'FontSize', 12 );
    set(ttext, ...
        'FontName', 'Palatino', ...
        'FontSize', 16, ...
        'FontWeight', 'bold');
    set(legnd, ...
        'FontName', 'Palatino', ...
        'FontSize', 12,...
        'Location', 'SouthWest',...
        'Box','off');
    filename = 'kpath';
    print(printtype, strcat(printpath, filename, '.', filetype),resolution);


    % --- C plot ---
    figure(13)
    hold on
    ttext = title('paths of C over time');
    xtext = xlabel('time');
    ytext = ylabel('C');
    line = plot(time, Cstar1 + 0*time,'k--'); %steady state line
    grid on

    for i = 1:Nsig
        set(Cplot(i), ...
            'Color', colors(i,:), ...
            'LineWidth', line_width, ...
            'LineStyle', '-');
    end
    legnd = legend(legendcell);
    set(xtext, ...
        'FontName', 'Palatino', ...
        'FontSize', 12 );
    set(ytext, ...
        'FontName', 'Palatino', ...
        'FontSize', 12 );
    set(ttext, ...
        'FontName', 'Palatino', ...
        'FontSize', 16, ...
        'FontWeight', 'bold');
    set(legnd, ...
        'FontName', 'Palatino', ...
        'FontSize', 12,...
        'Location', 'SouthWest',...
        'Box','off');
    filename = 'Cpath';
    print(printtype, strcat(printpath, filename, '.', filetype),resolution);


    % --- tax plot ---
    figure(14)
    hold on
    ttext = title('paths of taxes over time');
    xtext = xlabel('time');
    ytext = ylabel('taxes');
    grid on
    if figno == 1 || figno == 2
        axis([xlim 0 0.1])
    else
        axis([xlim -0.1 0.1])
    end

    for i = 1:Nsig
        set(taxplot(i), ...
            'Color', colors(i,:), ...
            'LineWidth', line_width, ...
            'LineStyle', '-');
    end
    legnd = legend(legendcell);
    set(xtext, ...
        'FontName', 'Palatino', ...
        'FontSize', 12 );
    set(ytext, ...
        'FontName', 'Palatino', ...
        'FontSize', 12 );
    set(ttext, ...
        'FontName', 'Palatino', ...
        'FontSize', 16, ...
        'FontWeight', 'bold');
    set(legnd, ...
        'FontName', 'Palatino', ...
        'FontSize', 12,...
        'Location', 'NorthEast',...
        'Box','off');
    filename = 'taxes';
    print(printtype, strcat(printpath, filename, '.', filetype),resolution);

    close all
end


%% Find tax = 1% times
if figno == 2
    taxtime = zeros(length(param_all),1);
    for param = 1:length(param_all)
        temp = find(TAXES(:,param)<0.01,1,'first');
        if isempty(temp) == 0
            taxtime(param) = temp;
        end
    end

    last_ind = max( [1:length(param_all)]' .* (taxtime>0) );

    % Show the times
    [param_all' taxtime]

    % Display it in tikZ style, to copy paste it into a tikz file
    tikztext = '';
    for param = 1:last_ind
        tikztext = [tikztext '(' num2str(param_all(param)) ',' num2str(taxtime(param)) ') '];
    end
    disp(tikztext)
end
